@@ -1,7 +1,7 @@
 /* Test file for mpfr_const_pi.
 
-Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
-Contributed by the Arenaire and Cacao projects, INRIA.
+Copyright 1999, 2001-2017 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -20,11 +20,72 @@ along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
 http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
+#if defined (WANT_SHARED_CACHE) && defined(HAVE_PTHREAD)
+
+# include <pthread.h>
+
+#define MAX_THREAD 100
+
+static void *
+start_routine (void *arg)
+{
+  mpfr_prec_t p;
+  mpfr_t x;
+  mpfr_prec_t inc = *(int *) arg;
+  mp_limb_t *m;
+
+  for (p = 100; p < 20000; p += 64 + 100 * (inc % 10))
+    {
+      mpfr_init2 (x, p);
+      m = MPFR_MANT (x);
+      mpfr_const_pi (x, MPFR_RNDD);
+      mpfr_prec_round (x, 53, MPFR_RNDD);
+      if (mpfr_cmp_str1 (x, "3.141592653589793116"))
+        {
+          printf ("mpfr_const_pi failed with threading\n");
+          mpfr_out_str (stdout, 10, 0, x, MPFR_RNDN); putchar('\n');
+          exit (1);
+        }
+      /* Check that no reallocation has been performed */
+      MPFR_ASSERTN (m == MPFR_MANT (x));
+      mpfr_clear (x);
+    }
+
+  pthread_exit (NULL);
+}
+
+static void
+run_pthread_test (void)
+{
+  int i;
+  int error_code;
+  pthread_t thread_id[MAX_THREAD];
+  int table[MAX_THREAD];
+
+  for (i = 0; i < MAX_THREAD; i++)
+    {
+      table[i] = i;
+      error_code = pthread_create(&thread_id[i],
+                                  NULL, start_routine, &table[i]);
+      MPFR_ASSERTN (error_code == 0);
+    }
+
+  for (i = 0; i < MAX_THREAD; i++)
+    {
+      error_code = pthread_join (thread_id[i], NULL);
+      MPFR_ASSERTN (error_code == 0);
+    }
+}
+
+# define RUN_PTHREAD_TEST()                                             \
+  (MPFR_ASSERTN(mpfr_buildopt_sharedcache_p() == 1), run_pthread_test())
+#else
+# define RUN_PTHREAD_TEST() \
+  (MPFR_ASSERTN(mpfr_buildopt_sharedcache_p() == 0))
+#endif
+
 /* tconst_pi [prec] [rnd] [0 = no print] */
 
 static void
@@ -87,13 +148,15 @@ bug20091030 (void)
       inex = mpfr_const_pi (x, MPFR_RNDU);
       if (inex < 0)
         {
-          printf ("Error, inex < 0 for RNDU (prec=%lu)\n", p);
+          printf ("Error, inex < 0 for RNDU (prec=%lu)\n",
+                  (unsigned long) p);
           exit (1);
         }
       inex = mpfr_const_pi (x, MPFR_RNDD);
       if (inex > 0)
         {
-          printf ("Error, inex > 0 for RNDD (prec=%lu)\n", p);
+          printf ("Error, inex > 0 for RNDD (prec=%lu)\n",
+                  (unsigned long) p);
           exit (1);
         }
     }
@@ -107,11 +170,13 @@ bug20091030 (void)
       mpfr_set_prec (x_ref, p);
       for (r = 0; r < MPFR_RND_MAX; r++)
         {
+          if (r == MPFR_RNDF)
+            continue; /* the test below makes no sense */
           inex = mpfr_const_pi (x, (mpfr_rnd_t) r);
           inex_ref = mpfr_const_pi_internal (x_ref, (mpfr_rnd_t) r);
           if (inex != inex_ref || mpfr_cmp (x, x_ref) != 0)
             {
-              printf ("mpfr_const_pi and mpfr_const_pi_internal disagree\n");
+              printf ("mpfr_const_pi and mpfr_const_pi_internal disagree for rnd=%s\n", mpfr_print_rnd_mode ((mpfr_rnd_t) r));
               printf ("mpfr_const_pi gives ");
               mpfr_dump (x);
               printf ("mpfr_const_pi_internal gives ");
@@ -138,7 +203,7 @@ main (int argc, char *argv[])
   if (argc > 1)
     {
       long a = atol (argv[1]);
-      if (a >= MPFR_PREC_MIN && a <= MPFR_PREC_MAX)
+      if (MPFR_PREC_COND (a))
         p = a;
     }
 
@@ -176,7 +241,9 @@ main (int argc, char *argv[])
 
   check_large ();
 
-  test_generic (2, 200, 1);
+  test_generic (MPFR_PREC_MIN, 200, 1);
+
+  RUN_PTHREAD_TEST();
 
   tests_end_mpfr ();
 
