@@ -1,7 +1,7 @@
 /* Test file for mpfr_set_str.
 
-Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
-Contributed by the Arenaire and Caramel projects, INRIA.
+Copyright 1999, 2001-2017 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -20,8 +20,6 @@ along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
 http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
 #define N 30000
@@ -31,11 +29,10 @@ http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
   mpfr_set_str_binary (x, t); \
   if (mpfr_cmp (x, y)) \
     { \
-      printf ("Error in mpfr_set_str (%d):\n", n); \
-      mpfr_print_binary (x); \
-      puts (""); \
-      mpfr_print_binary (y); \
-      puts (""); \
+      printf ("Error in mpfr_set_str (%d):\nexpected ", n); \
+      mpfr_dump (x); \
+      printf ("got      "); \
+      mpfr_dump (y); \
       mpfr_clear (x); \
       mpfr_clear (y); \
       exit (1); \
@@ -66,7 +63,7 @@ check_underflow (void)
 
   /* check overflow */
   emax = mpfr_get_emax ();
-  set_emax (1073741823); /* 2^30-1 */
+  set_emax (1073741821); /* 2^30-3 */
   mpfr_set_str (a, "2E1000000000", 10, MPFR_RNDN);
   if (!mpfr_inf_p (a) || mpfr_sgn (a) < 0)
     {
@@ -100,9 +97,8 @@ int
 main (int argc, char *argv[])
 {
   mpfr_t x, y;
-  unsigned long k, bd, nc, i;
-  char *str, *str2;
-  mpfr_exp_t e;
+  long nc, i;
+  char *str;
   int base, logbase, prec, baseprec, ret, obase;
 
   tests_start_mpfr ();
@@ -120,34 +116,59 @@ main (int argc, char *argv[])
       return 0;
     }
 
-  mpfr_init2 (x, 2);
-
   nc = (argc > 1) ? atoi(argv[1]) : 53;
   if (nc < 100)
     nc = 100;
 
-  bd = randlimb () & 8;
+  str = (char *) tests_allocate (nc);
 
-  str2 = str = (char*) (*__gmp_allocate_func) (nc);
+  mpfr_init2 (x, nc + 10);
 
-  if (bd)
+#define NR 50
+
+  for (i = 0; i < NR; i++)
     {
-      for(k = 1; k <= bd; k++)
-        *(str2++) = (randlimb () & 1) + '0';
+      char *str2 = str;
+      long bd, k, lz;
+
+      bd = randlimb () & 8;  /* 0 or 8 */
+      lz = -bd;
+
+      if (bd)
+        {
+          for (k = 1; k <= bd; k++, str2++)
+            {
+              *str2 = '0' + (randlimb () & 1);
+              if (lz == -bd && *str2 != '0')
+                lz = k - bd; /* position of the first 1 */
+            }
+        }
+      else
+        *(str2++) = '0';
+
+      *(str2++) = '.';
+
+      for (k = 1; k < nc - 17 - bd; k++, str2++)
+        {
+          *str2 = '0' + (randlimb () & 1);
+          if (lz == -bd && *str2 != '0')
+            lz = k; /* position of the first 1 */
+        }
+
+      *(str2++) = 'e';
+
+      /* Half cases have an exponent around zero, the other half cases
+         have the minimum exponent for which the value is representable
+         (not a subnormal). */
+      sprintf (str2, "%" MPFR_EXP_FSPEC "d", i < NR/2 ?
+               ((mpfr_eexp_t) (randlimb () & 0xff) - 0x80) :
+               ((mpfr_eexp_t) mpfr_get_emin () + lz - 1));
+
+      /* if (i >= NR/2) printf ("lz = %ld, str: %s\n", lz, str); */
+      mpfr_set_str_binary (x, str);
     }
-  else
-    *(str2++) = '0';
 
-  *(str2++) = '.';
-
-  for (k = 1; k < nc - 17 - bd; k++)
-    *(str2++) = '0' + (char) (randlimb () & 1);
-
-  *(str2++) = 'e';
-  sprintf (str2, "%d", (int) (randlimb () & INT_MAX) + INT_MIN/2);
-
-  mpfr_set_prec (x, nc + 10);
-  mpfr_set_str_binary (x, str);
+  tests_free (str, nc);
 
   mpfr_set_prec (x, 54);
   mpfr_set_str_binary (x, "0.100100100110110101001010010101111000001011100100101010E-529");
@@ -156,10 +177,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (1a):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -170,17 +189,13 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (1b):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
     }
 
-  (*__gmp_free_func) (str, nc);
-
   mpfr_set_prec (x, 53);
   mpfr_set_str_binary (x, "+110101100.01010000101101000000100111001000101011101110E00");
 
@@ -214,9 +229,10 @@ main (int argc, char *argv[])
   prec = 53;
   mpfr_set_prec (x, prec);
   mpfr_set_prec (y, prec);
-  for (i=0;i<N;i++)
+  for (i = 0; i < N; i++)
     {
       mpfr_rnd_t rnd;
+      mpfr_exp_t e;
 
       mpfr_urandomb (x, RANDS);
       rnd = RND_RAND ();
@@ -232,23 +248,22 @@ main (int argc, char *argv[])
         baseprec = 1 + (prec - 2 + logbase) / logbase;
       str = mpfr_get_str (NULL, &e, base, baseprec, x, rnd);
       mpfr_set_str (y, str, base, rnd);
-      MPFR_EXP(y) += logbase * (e - strlen (str));
+      if (!MPFR_IS_ZERO(y))
+        MPFR_EXP(y) += logbase * (e - strlen (str));
       if (mpfr_cmp (x, y))
         {
           printf ("mpfr_set_str o mpfr_get_str <> id for rnd_mode=%s\n",
                   mpfr_print_rnd_mode (rnd));
           printf ("x=");
-          mpfr_print_binary (x);
-          puts ("");
+          mpfr_dump (x);
           printf ("s=%s, exp=%d, base=%d\n", str, (int) e, base);
           printf ("y=");
-          mpfr_print_binary (y);
-          puts ("");
+          mpfr_dump (y);
           mpfr_clear (x);
           mpfr_clear (y);
           exit (1);
         }
-      (*__gmp_free_func) (str, strlen (str) + 1);
+      mpfr_free_str (str);
     }
 
   for (i = 2; i <= 62; i++)
@@ -262,21 +277,21 @@ main (int argc, char *argv[])
 
       /*
       if (mpfr_set_str (x, "@Inf@garbage", i, MPFR_RNDN) != 0 ||
-          !mpfr_inf_p(x) || MPFR_SIGN(x) < 0)
+          !mpfr_inf_p(x) || MPFR_IS_NEG (x))
         {
           printf ("mpfr_set_str failed on @Inf@garbage\n");
           exit (1);
         }
 
       if (mpfr_set_str (x, "-@Inf@garbage", i, MPFR_RNDN) != 0 ||
-          !mpfr_inf_p(x) || MPFR_SIGN(x) > 0)
+          !mpfr_inf_p(x) || MPFR_IS_POS (x))
         {
           printf ("mpfr_set_str failed on -@Inf@garbage\n");
           exit (1);
         }
 
       if (mpfr_set_str (x, "+@Inf@garbage", i, MPFR_RNDN) != 0 ||
-          !mpfr_inf_p(x) || MPFR_SIGN(x) < 0)
+          !mpfr_inf_p(x) || MPFR_IS_NEG (x))
         {
           printf ("mpfr_set_str failed on +@Inf@garbage\n");
           exit (1);
@@ -294,21 +309,21 @@ main (int argc, char *argv[])
         }
 
       if (mpfr_set_str (x, "Inf", i, MPFR_RNDN) != 0 ||
-          !mpfr_inf_p(x) || MPFR_SIGN(x) < 0)
+          !mpfr_inf_p(x) || MPFR_IS_NEG (x))
         {
           printf ("mpfr_set_str failed on Inf\n");
           exit (1);
         }
 
       if (mpfr_set_str (x, "-Inf", i, MPFR_RNDN) != 0 ||
-          !mpfr_inf_p(x) || MPFR_SIGN(x) > 0)
+          !mpfr_inf_p(x) || MPFR_IS_POS (x))
         {
           printf ("mpfr_set_str failed on -Inf\n");
           exit (1);
         }
 
       if (mpfr_set_str (x, "+Inf", i, MPFR_RNDN) != 0 ||
-          !mpfr_inf_p(x) || MPFR_SIGN(x) < 0)
+          !mpfr_inf_p(x) || MPFR_IS_NEG (x))
         {
           printf ("mpfr_set_str failed on +Inf\n");
           exit (1);
@@ -334,10 +349,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (2):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -355,10 +368,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (3):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -379,10 +390,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (4):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -398,10 +407,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (5):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -415,10 +422,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (6):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -432,10 +437,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (7):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -450,10 +453,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (8):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -464,10 +465,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (9):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -478,10 +477,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (10):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -492,10 +489,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (11):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -506,10 +501,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (12):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -520,10 +513,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (13):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -534,10 +525,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (14):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -548,10 +537,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (15):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -562,10 +549,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (16):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -576,10 +561,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (17):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -590,10 +573,8 @@ main (int argc, char *argv[])
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (18):\n");
-      mpfr_print_binary (x);
-      puts ("");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (x);
+      mpfr_dump (y);
       mpfr_clear (x);
       mpfr_clear (y);
       exit (1);
@@ -762,7 +743,7 @@ main (int argc, char *argv[])
     mpfr_set_prec (x, mp_bits_per_limb); /* x and y have only one limb */
     mpfr_set_prec (y, mp_bits_per_limb);
 
-    str = (char*) (*__gmp_allocate_func) (N + 20);
+    str = (char *) tests_allocate (N + 20);
 
     mpfr_set_ui (x, 1, MPFR_RNDN); /* ensures that x is not NaN or Inf */
     for (; nb_digit < N; nb_digit *= 10)
@@ -794,30 +775,29 @@ main (int argc, char *argv[])
                           "rnd=%s:\n", (unsigned int) nb_digit, base[cbase],
                           mpfr_print_rnd_mode (rnd[crnd]));
                   printf ("instead of: ");
-                  mpfr_print_binary (x);
-                  puts ("");
+                  mpfr_dump (x);
                   printf ("return    : ");
-                  mpfr_print_binary (y);
-                  puts ("");
+                  mpfr_dump (y);
                   exit (1);
                 }
             }
 
-    (*__gmp_free_func) (str, N + 20);
+    tests_free (str, N + 20);
   }
 
   /* end of tests added by Alain Delplanque */
 
-  /* check that flags are correctly cleared */
   mpfr_set_nan (x);
   mpfr_set_str (x, "+0.0", 10, MPFR_RNDN);
-  if (!mpfr_number_p(x) || mpfr_cmp_ui (x, 0) != 0 || mpfr_sgn (x) < 0)
+  if (MPFR_NOTZERO (x) || MPFR_IS_NEG (x))
     {
       printf ("x <- +0.0 failed after x=NaN\n");
       exit (1);
     }
+
+  mpfr_set_nan (x);
   mpfr_set_str (x, "-0.0", 10, MPFR_RNDN);
-  if (!mpfr_number_p(x) || mpfr_cmp_ui (x, 0) != 0 || mpfr_sgn (x) > 0)
+  if (MPFR_NOTZERO (x) || MPFR_IS_POS (x))
     {
       printf ("x <- -0.0 failed after x=NaN\n");
       exit (1);
